/*
 * The Clear BSD License
 * Copyright (c) 2013 - 2015, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted (subject to the limitations in the disclaimer below) provided
 *  that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE GRANTED BY THIS LICENSE.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "fsl_device_registers.h"
#include "fsl_debug_console.h"
#include "board.h"
#include "fsl_sctimer.h"
#include "pin_mux.h"
#include "cmd_line.h"
#include "uart.h"

#include <stdbool.h>
#include <stdio.h>

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define SCTIMER_CLK_FREQ          CLOCK_GetFreq(kCLOCK_Irc)

/* Set the frequency at which we will timeout waiting for an input */
#define PERIOD_TIMEOUT_FREQ_HZ    2
#define PERIOD_TIMEOUT_TICKS      ((SystemCoreClock / PERIOD_TIMEOUT_FREQ_HZ) - 1)

#define COM_UART UART1

/*******************************************************************************
 * Prototypes
 ******************************************************************************/
uint32_t          oldCaptureReg;
volatile uint32_t oldCaptureVal;
uint32_t          newCaptureReg;
volatile uint32_t newCaptureVal;
uint32_t          period;
volatile uint8_t  periodAvailable;

uint32_t          initialRisingEdgeInput;
uint32_t          finalRisingEdgeInput;
uint32_t          timeout;

void SCT_Initial_Detection_Callback(void)
{
    oldCaptureVal   = SCT0->SCTCAP[oldCaptureReg];
    periodAvailable = 0;

    SCTIMER_EnableInterrupts(SCT0, 1 << timeout);

    /* Reload the timeout match register */
    SCT0->CTRL             |= (SCT_CTRL_HALT_L_MASK);
    SCT0->SCTMATCH[timeout] = (PERIOD_TIMEOUT_TICKS + SCT0->COUNT);
    SCT0->CTRL             &= ~(SCT_CTRL_HALT_L_MASK);
}

void SCT_Final_Detection_Callback(void)
{
    newCaptureVal      = SCT0->SCTCAP[newCaptureReg];
    period             = newCaptureVal - oldCaptureVal;
    oldCaptureVal      = newCaptureVal;
    periodAvailable    = 1;

    /* Reload the timeout match register */
    SCT0->CTRL             |= (SCT_CTRL_HALT_L_MASK);
    SCT0->SCTMATCH[timeout] = (PERIOD_TIMEOUT_TICKS + SCT0->COUNT);
    SCT0->CTRL             &= ~(SCT_CTRL_HALT_L_MASK);
}

void SCT_Timeout_Callback(void)
{
    periodAvailable = 0;

    SCTIMER_DisableInterrupts(SCT0, 1 << timeout);

    /* Reload the timeout match register */
    SCT0->CTRL             |= (SCT_CTRL_HALT_L_MASK);
    SCT0->SCTMATCH[timeout] = (PERIOD_TIMEOUT_TICKS + SCT0->COUNT);
    SCT0->CTRL             &= ~(SCT_CTRL_HALT_L_MASK);
}

/*******************************************************************************
 * Code
 ******************************************************************************/

char outputBuf[80];

int main(void)
{
    char                       ch;
    sctimer_config_t           sctimerInfo;
    sctimer_pwm_signal_param_t pwmParam;
    uint32_t                   event;
    float                      freq;

    /* Init board hardware. */

    BOARD_InitPins();
    /* Do not init the clocks here. Rowley has done this in system_LPC8xx.c
     *
     * BOARD_InitBootClocks();
     *
     */

    uartClockSetup(3686400);
    uartInit(UART0, 115200, UART_8N1, 2);
    uartInit(UART1, 115200, UART_8N1, 2);

    SCTIMER_GetDefaultConfig(&sctimerInfo);

    /* Initialize SCTimer module */
    SCTIMER_Init(SCT0, &sctimerInfo);

    /* Set the INSYNC bit for input 0 */
    SCT0->CONFIG |= (1 << (kSCTIMER_Input_0 + SCT_CONFIG_INSYNC_SHIFT));

    /* Schedule an event to look for a timeout waiting for the input 0 */
    if (SCTIMER_CreateAndScheduleEvent(SCT0,
                                       kSCTIMER_MatchEventOnly,
                                       (PERIOD_TIMEOUT_TICKS + SCT0->COUNT),
                                       0,
                                       kSCTIMER_Counter_L,
                                       &timeout) == kStatus_Fail)
    {
        return(-1);
    }

    /* Schedule an event to look for a rising edge on input 0 in state 0 */
    if (SCTIMER_CreateAndScheduleEvent(SCT0,
                                       kSCTIMER_InputRiseEvent,
                                       0,
                                       kSCTIMER_Input_0,
                                       kSCTIMER_Counter_L,
                                       &initialRisingEdgeInput) == kStatus_Fail)
    {
        return(-1);
    }

    if (SCTIMER_SetupCaptureAction(SCT0,
                                   kSCTIMER_Counter_L,
                                   &oldCaptureReg,
                                   initialRisingEdgeInput) == kStatus_Fail)
    {
        return(-1);
    }

    /* Transition to state 1 when a rising edge is detected on input 0 in state 0 */
    SCTIMER_SetupNextStateAction(SCT0, 1, initialRisingEdgeInput);

    /* Go to next state; State 1 */
    if (SCTIMER_IncreaseState(SCT0) == kStatus_Fail)
    {
        return(-1);
    }


    /* Schedule the timeout event in state 1 as well */
    SCTIMER_ScheduleEvent(SCT0, timeout);

    /* Transition back to state 0 when a timeout is detected */
    SCTIMER_SetupNextStateAction(SCT0, 0, timeout);

    /* Schedule an event to look for rising edge on input 0 in state 1 */
    if (SCTIMER_CreateAndScheduleEvent(SCT0,
                                       kSCTIMER_InputRiseEvent,
                                       0,
                                       kSCTIMER_Input_0,
                                       kSCTIMER_Counter_L,
                                       &finalRisingEdgeInput) == kStatus_Fail)
    {
        return(-1);
    }

    if (SCTIMER_SetupCaptureAction(SCT0,
                                   kSCTIMER_Counter_L,
                                   &newCaptureReg,
                                   finalRisingEdgeInput) == kStatus_Fail)
    {
        return(-1);
    }

    /* Set the callback funcitons */
    SCTIMER_SetCallback(SCT0, SCT_Initial_Detection_Callback, initialRisingEdgeInput);
    SCTIMER_SetCallback(SCT0, SCT_Final_Detection_Callback, finalRisingEdgeInput);
    SCTIMER_SetCallback(SCT0, SCT_Timeout_Callback, timeout);

//    /* Halt the timer when any of the events occur */
//    SCTIMER_SetupCounterHaltAction(SCT0, kSCTIMER_Counter_L, initialRisingEdgeInput);
//    SCTIMER_SetupCounterHaltAction(SCT0, kSCTIMER_Counter_L, finalRisingEdgeInput);
//    SCTIMER_SetupCounterHaltAction(SCT0, kSCTIMER_Counter_L, timeout);

    /* Enable interrupts for all events */
    SCTIMER_EnableInterrupts(SCT0, 1 << initialRisingEdgeInput);
    SCTIMER_EnableInterrupts(SCT0, 1 << finalRisingEdgeInput);
    SCTIMER_EnableInterrupts(SCT0, 1 << timeout);

    /* Toggle the output when either input event is received */
    SCTIMER_SetupOutputToggleAction(SCT0,
                                    kSCTIMER_Out_0,
                                    initialRisingEdgeInput);

    SCTIMER_SetupOutputToggleAction(SCT0,
                                    kSCTIMER_Out_0,
                                    finalRisingEdgeInput);

    /* Enable the SCT0 interrupt */
    NVIC_EnableIRQ(SCT0_IRQn);

    /* Start the timer */
    SCTIMER_StartTimer(SCT0, kSCTIMER_Counter_L);

    periodAvailable = 0;
    
    cmdParserInit();

    uartPutStr(COM_UART, "Measuring the frequency of the input waveform\r\n");

    while (1)
    {
        if (periodAvailable)
        {
            freq = (float)SystemCoreClock / (float)(period);
            sprintf(outputBuf, "freq = %f\r\n", freq);
            uartWrite(COM_UART, outputBuf, strlen(outputBuf));

            if (freq > 10000)
            {
                uartPutStr(COM_UART, "freq error\r\n");
            }
            periodAvailable = 0;
        }
        else
        {
//            printf("freq unavailable\r\n");
        }
        cmdProcess();
        __WFI();
    }
} /* main */

